/*****************************************************************************
*  Copyright (c) 2005 Digi International Inc., All Rights Reserved
*
*  This software contains proprietary and confidential information of Digi
*  International Inc.  By accepting transfer of this copy, Recipient agrees
*  to retain this software in confidence, to prevent disclosure to others,
*  and to make no use of this software other than that for which it was
*  delivered.  This is an unpublished copyrighted work of Digi International
*  Inc.  Except as permitted by federal law, 17 USC 117, copying is strictly
*  prohibited.
*
*  Restricted Rights Legend
*
*  Use, duplication, or disclosure by the Government is subject to
*  restrictions set forth in sub-paragraph (c)(1)(ii) of The Rights in
*  Technical Data and Computer Software clause at DFARS 252.227-7031 or
*  subparagraphs (c)(1) and (2) of the Commercial Computer Software -
*  Restricted Rights at 48 CFR 52.227-19, as applicable.
*
*  Digi International Inc. 11001 Bren Road East, Minnetonka, MN 55343
*
*****************************************************************************/

#ifndef NA_STATUS_H_
#define NA_STATUS_H_

#ifdef __cplusplus
extern "C"
{
#endif

/*ad*
 * Return status from the API functions. 
 *
 * NaStatus is a general purpose return code that many functions return to indicate their completion       
 * status. An NaStatus code can indicate general success, a specific error, or a warning. The code can     
 * contain an indication of what component triggered the error so that as NaStatus codes are bubbled up    
 * through multiple API layers the root of the error is still be identifiable.                             
 *                                                                                                         
 * <example>
 *
 * NaStatus is a 4 byte value broken down as follows:                                                      
 *       | 3 3  | 2 2 | 2 2 2 2 2 2 2 2 1 1 1 1 | 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 |
 *       | 1 0  | 9 8 | 7 6 5 4 3 2 1 0 9 8 7 6 | 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 |
 *       | Type | Res |         Facility        |             Code                |
 *                                                                                                         
 * where:                                                                                                  
 *   Type: These bits indicate the general nature of the status code (ie success, warning, error). The values are:         
 *      00 - Success: The operation completed normally
 *      01 - Warning: The operation completed successfully but it is signalling a warning condition to the caller
 *      10 - Error:   The operation did not complete successfully
 *   Res: These bits are reserved for future use
 *   Facility: These bits indicate what OS compoent originated the status code. The range of facility codes is:                            
 *      0x000-0x3FF  - NET OS components
 *      0x400-0x4FF  - Reserved 
 *      0x800-0xFFF  - User components (note MSB indicates a user component)
 *   Code: The unique code indicating what specifically happened. These values can range from 0x00-0xFF and are defined by the
 *         facility omitting the error.                                           
 *                                                                                                         
 * NaStatus codes are defined by combining these elements together into a constant that can be referred to by 
 * code returning the status as well as by code receiving the status.      
 *                                
 * The following Macros are provided to assist you in examining the type of an NaStatus value:
 *   NA_IS_ERROR(x)    returns true if status x is of type error
 *   NA_IS_WARNING(x)  returns true if status x is of type warning
 *   NA_IS_SUCCESS(x)  returns true if status x is of type success OR warning
 *
 * Example use:
 *   NaStatus s;
 *   s = naDoMagic();                // call an API function returning NaStatus
 *   if (NA_IS_ERROR(s)) {           // check for any kind of error
 *      if (s == NASTATUS_IN_USE) {  // check for a specific error
 *          ... try again ...
 *      } else {
 *         ... return error ...
 *      }
 *   }
 *
 * </example>
 *
 * @external
 * @category general_purpose
 * @since 6.2
 * @include "nastatus.h"
 *
 * @see @link NaStatusValues
 *
*ad*/
typedef unsigned long NaStatus;

// The constants below define the Type indicator mask and values
#define NASTATUS_TYPE_MASK             0xC0000000UL
#define NASTATUS_TYPE_OFFSET           30
#define NASTATUS_WARNING               ((0x1UL << NASTATUS_TYPE_OFFSET))
#define NASTATUS_ERROR                 ((0x2UL << NASTATUS_TYPE_OFFSET))

// The constants below define the Facility mask and values
#define NASTATUS_FAC_MASK             0x0FFF0000UL
#define NASTATUS_FAC_OFFSET           16
// NETOS facilities
#define NASTATUS_GEN                  (0x001UL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_PD                   (0x002UL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_IPSEC                (0x003UL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_TCPIP                (0x004UL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_EKAHAU               (0x007UL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_SNMP                 (0x008UL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_IF                   (0x009UL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_AES                  (0x00AUL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_A2D                  (0x00BUL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_SCRP                 (0x00CUL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_CMPR                 (0x00EUL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_GFX                  (0x010UL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_LCD                  (0x011UL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_SM                   (0x012UL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_RTC                  (0x013UL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_ETH                  (0x014UL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_QUAD                 (0x015UL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_SPI_MASTER           (0x016UL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_SPI_SLAVE            (0x017UL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_IOP                  (0x018UL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_FAT2SDIO             (0x019UL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_SDIODRVR             (0x01AUL  << NASTATUS_FAC_OFFSET)
#define NASTATUS_CBUS                 (0x01CUL  << NASTATUS_FAC_OFFSET)

// Note: The NDS facilities are defined in ndsstatus.h
// Note: User facilities: must start at NASTATUS_USER
#define NASTATUS_USER                 (0X800UL  << NASTATUS_FAC_OFFSET)


// Status Checking Helper Macros 
#define NA_IS_ERROR(status)    (((long)status)<0)
#define NA_IS_WARNING(status)  ((status & NASTATUS_TYPE_MASK) == NASTATUS_WARNING)
#define NA_IS_SUCCESS(status)  (!NA_IS_ERROR(status))
#define NA_STATUS_COMP(status) ((status & NASTATUS_FAC_MASK) >> NASTATUS_FAC_OFFSET)



/*ad*
 * API Return Values
 *
 * While it is possible for API functions to return a status of NASTATUS_SUCCESS, NASTATUS_WARNING, or NASTATUS_ERROR, it
 * is also common for them to return more specific status values that fall under one of those types. To check for
 * the general type of a status code you should use the helper macro's NA_IS_ERROR(), NA_IS_WARNING(), or NA_IS_SUCCESS().
 * Refer to NaStatus help for additional detail on the use of those macros. 
 *
 * In addition, you can test for the following specific error codes that can be returned
 * by various API functions.
 *
 * @param NASTATUS_SUCCESS                   This value indicates the operation completed successfully.
 * @param NASTATUS_INVALID_CHANNEL           Generic status error for invalid channel.
 * @param NASTATUS_INVALID_HANDLE            Generic status error for invalid handle.
 * @param NASTATUS_INVALID_PARAMETER         Generic status error for invalid parameter.
 * @param NASTATUS_RESOURCE_ERROR            Generic status error for resource.
 * @param NASTATUS_IN_USE                    Generic status error for in use 
 * @param NASTATUS_MALLOC_ERROR              Generic status error for out of memory.
 * @param NASTATUS_NOT_INITIALIZED           Generic status error for device not initialized
 * @param NASTATUS_ENTRY_NOT_FOUND           Generic status error for missing entry   
 * @param NASTATUS_PD_OUT_OF_RESOURCES       Mode was set to NAPD_DEEP_SLEEP but the external hardware needed to support this mode was not detected
 * @param NASTATUS_PD_INVALID_TRIGGER        Unsupported wakeup trigger is selected
 * @param NASTATUS_PD_NO_TRIGGER             Mode is set to NAPD_SLEEP_AND_RESET or NAPD_SLEEP_AND_RESUME but wake trigger was set to zero
 * @param NASTATUS_PD_NULL_POINTER           NULL function pointer was passed to register to call before power save mode
 * @param NASTATUS_PD_BAD_PORT               Bad serial port is selected
 * @param NASTATUS_PD_BAD_DIVISOR            One or both of the clock divisors is invalid 
 * @param NASTATUS_PD_BAD_MODE               naPdSleep() is called with invalid mode
 * @param NASTATUS_PD_FAILED_TO_SET_INTR     Either failed to disable an interrupt or failed to enable back
 * @param NASTATUS_PD_ALREADY_IN_CATNAP      naPdCatnapStart() is called back to back
 * @param NASTATUS_PD_WATCHDOG_ERROR         NAPD_SLEEP_AND_RESET mode was set, watchdog timer is enabled and 
 *                                           the function failed to switch the watchdog to interrupt mode 
 * @param NASTATUS_IPSEC_OUT_OF_RESOURCES    Insufficient memory to allocate needed buffers
 * @param NASTATUS_IPSEC_INVALID_PARAMETER   One or more function parameters are invalid
 * @param NASTATUS_IPSEC_NOT_INITIALIZED     IPSEC or IKE is not initialized
 * @param NASTATUS_IPSEC_INVALID_PEER        The peer is not valid
 * @param NASTATUS_IPSEC_INVALID_KEY         The key is not valid
 * @param NASTATUS_IPSEC_INVALID_ENCRYPTION_ALGORITHM       The encryption algorith specified is not valid
 * @param NASTATUS_IPSEC_INVALID_KEYLENGTH                  The key length is not valid for the algorithm specified
 * @param NASTATUS_IPSEC_INVALID_AUTHENTICATION_ALGORITHM   The authentication algorithm is not valid
 * @param NASTATUS_IPSEC_INVALID_HANDLE      The handle is not valid
 * @param NASTATUS_IPSEC_INVALID_SECONDS     The lifetime in seconds is out of range
 * @param NASTATUS_IPSEC_INVALID_KILOBYTES   The lifetime in kilobytes is out of range
 * @param NASTATUS_IPSEC_INVALID_INDEX       The index is out of range
 * @param NASTATUS_IPSEC_WRONG_LIBRARY       IPSEC is not built into this version of the TCP/IP library
 * @param NASTATUS_IPSEC_ALREADY_INITIALIZED IPSEC has already been initialized
 * @param NASTATUS_IPSEC_INVALID_DEFAULT_POLICY             The default policy does not select all packets, or does not either discard them or pass them onto TCP/IP.
 * @param NASTATUS_IPSEC_INVALID_SELECTOR    The selector handle is not valid
 * @param NASTATUS_IPSEC_INVALID_ADDRESS     The address is not valid
 * @param NASTATUS_IPSEC_INVALID_IP_ADDRESS  The IP address is not valid
 * @param NASTATUS_IPSEC_SA_NOT_ACTIVE       The SA has not been activated yet
 * @param NASTATUS_IPSEC_INVALID_TABLE_ENTRY An entry in the table is invalid
 * @param NASTATUS_IPSEC_INVALID_RULES       Conflicting rules have been specified
 * @param NASTATUS_IPSEC_SA_IN_USE           The SA has already been actived and cannot be changed until it is deactived
 * @param NASTATUS_IPSEC_NOT_CONFIGURED      The object is not properly configured
 * @param NASTATUS_TCPIP_CANNOT_ADD_ROUTE    Error while adding new entry to routing table
 * @param NASTATUS_TCPIP_ARP_CACHE_FULL      Arp cache table is full 
 * @param NASTATUS_EKAHAU_NO_MEMORY          Out of memory
 * @param NASTATUS_SNMP_INVALID_POINTER       Invalid data pointer passed in the API
 * @param NASTATUS_SNMP_NO_MEMORY             Low memory condition.
 * @param NASTATUS_SNMP_NOSUCH_TRAPADDR       There is no such address for deleting.
 * @param NASTATUS_SNMP_CANNOT_SEND_TRAP      Trap could not be generated
 * @param NASTATUS_SNMP_INVALID_DEST_PORT     Invalid trap sink destination port
 * @param NASTATUS_SNMP_GENERAL_ERROR         Unspecified error
 * @param NASTATUS_SNMP_TRAPS_MAX_REACHED     The maximum number of trap sinks has been reached
 * @param NASTATUS_SNMP_AGENT_LOCKED          Agent is currently locked
 * @param NASTATUS_SNMP_INVALID_SYSOID        Invalid system object ID
 * @param NASTATUS_SNMP_INVALID_SYSDESCR      Invalid system description
 * @param NASTATUS_SNMP_AGENT_STARTED         Agent has already been started
 * @param NASTATUS_SNMP_INVALID_COMMUNITY     Invalid community name or community length
 * @param NASTATUS_SNMP_INVALID_SYSSERV       Invalid system service value
 * @param NASTATUS_SNMP_AGENT_CANNOT_START    Failed to initialize agent
 * @param NASTATUS_SNMP_INVALID_ENTERPRISE    Invalid enterprise or enterprise length
 * @param NASTATUS_SNMP_INVALID_SYSCONTACT    Invalid system contact string
 * @param NASTATUS_SNMP_INVALID_SYSLOCATION   Invalid system location string
 * @param NASTATUS_SNMP_INVALID_SYSNAME       Invalid system name string
 * @param NASTATUS_SNMP_INVALID_ARGUMENT      One or more of the arguments is invalid
 * @param NASTATUS_IF_INVALID_IF_NAME         No network device with this name
 * @param NASTATUS_IF_ALREADY_CONFIGURED      Interface with the same name already exists
 * @param NASTATUS_IF_INTERNAL_ERROR          Lack of resources to complete operation
 * @param NASTATUS_IF_INVALID_LINK_TYPE       Unsupported link type
 * @param NASTATUS_IF_INVALID_PARAMETER       One or more of the arguments is invalid
 * @param NASTATUS_IF_ADDR_CONFLICT           IP address mapping found in ARP cache
 * @param NASTATUS_IF_NOT_CONFIGURED          Interface not configured with specified IP address
 * @param NASTATUS_IF_CONFIGURATION_IN_PROGRESS   Success: Interface configuration is in progress
 * @param NASTATUS_AES_MUTEX_FAILED           Call to mutex function returned error
 * @param NASTATUS_AES_INSTALL_ISR_FAILED     Call to naIsrInstall() failed
 * @param NASTATUS_AES_ALREADY_INIT           Aes DMA driver already initialized
 * @param NASTATUS_AES_NULL_POINTER           NULL pointer was passed in
 * @param NASTATUS_AES_INVALID_OPERATION      Invalid AES operation
 * @param NASTATUS_AES_ZERO_LENGTH_BUFFER     Zero length buffer was passed in
 * @param NASTATUS_AES_SEMAPHORE_FAILED       Call to semaphore function returned error
 * @param NASTATUS_AES_INVALID_DMA_DESC_INDEX Invalid DMA descriptor index
 * @param NASTATUS_AES_INVALID_KEY_SIZE       AES key size has to be 128, 192 or 256 bits
 * @param NASTATUS_AES_NOT_INIT               AES not initialized
 * @param NASTATUS_AES_INVALID_CCM_L_VALUE    CCM mode length field (L value) size must between 2 to 8 octets
 * @param NASTATUS_AES_INVALID_CCM_M_VALUE    CCM mode authentication field (M value) size must be 4, 6, 8, 10, 12, 14 or 16 octets
 * @param NASTATUS_AES_MALLOC_FAILED          Call to memory allocation function failed
 * @param NASTATUS_AES_CBUFFER_FULL           Circular buffer is full
 * @param NASTATUS_AES_OPENSSL_FUNCS_FAILED   Registering OpenSSL functions that use the AES hardware failed.
 * @param NASTATUS_AES_INVALID_BUFFER_LENGTH  Buffer length is not a multiple of 16
 * @param NASTATUS_AES_UNALIGNED_BUFFER       Buffer address is not on a 4 byte boundary
 * @param NASTATUS_AES_MAX_XFER_SIZE_EXCEEDED  The transfer size is larger than maximum DMA transfer size of 65520 bytes
 * @param NASTATUS_A2D_BAD_ARGUMENT           One or more function arguments is invalid
 * @param NASTATUS_A2D_BAD_CLOCK_RATE         The H/W does not support the clock rate
 * @param NASTATUS_A2D_BAD_HANDLE             The handle is invalid
 * @param NASTATUS_A2D_BAD_INPUT              The input number is out of range
 * @param NASTATUS_A2D_BAD_LENGTH             The length value is invalid
 * @param NASTATUS_A2D_BUFFER_Q_FULL          The buffer queue is full
 * @param NASTATUS_A2D_CHANNEL_IN_USE         The channel cannot be configured while automatic scanning is active
 * @param NASTATUS_A2D_DUPLICATE_CALLBACK     The notification function has already been registered
 * @param NASTATUS_A2D_DUPLICATE_ID           A channel with the same ID has already been registered
 * @param NASTATUS_A2D_INVALID_STATE          The operation is invalid given the channel's current state
 * @param NASTATUS_A2D_ISR_ALREADY_INSTALLED  The API only supports one ISR callback at a time, and it has already been installed
 * @param NASTATUS_A2D_ISR_NOT_INSTALLED      No ISR callback was installed
 * @param NASTATUS_A2D_NO_BUFFERS             There are no buffers in the queue
 * @param NASTATUS_A2D_NO_SUCH_CALLBACK       No such callback was registered
 * @param NASTATUS_A2D_NOT_AVAILABLE          The channel has already been opened
 * @param NASTATUS_A2D_NOT_CONFIGURED         The channel has not been configured
 * @param NASTATUS_A2D_NOT_RUNNING            Automatic scanning is not running
 * @param NASTATUS_A2D_NOT_SUPPORTED          The feature is not supported by this H/W
 * @param NASTATUS_A2D_OUT_OF_RESOURCES       The operation cannot be completed because the system is out of memory or some other resource
 * @param NASTATUS_A2D_UNKNOWN_IDENTIFIER     The identifier does not match any registered channel
 * @param NASTATUS_A2D_INCORRECT_VERSION      The structVersion field in the configuration structure is not set to the correct value
 * @param NASTATUS_SCRATCHPAD_BAD_ARGUMENT    Invalid argument
 * @param NASTATUS_SCRATCHPAD_BAD_SIZE        The given size is invalid
 * @param NASTATUS_SCRATCHPAD_DUPLICATE_LOCATION Another scratchpad device has the same bus type and location
 * @param NASTATUS_SCRATCHPAD_ARRAY_TOO_SMALL There are more scratchpad's devices
 * @param NASTATUS_SCRATCHPAD_UNKNOWN         The bus type and location do not match any known scratchpad's devices
 * @param NASTATUS_SCRATCHPAD_INVALID_HANDLE  The handle is invalid
 * @param NASTATUS_SCRATCHPAD_OUT_OF_BOUNDS   The offset and number of bytes to be read or write indicate an area outside 
 *                                            of the scratchpad's memory
 * @param NASTATUS_SCRATCHPAD_FAILURE         An error occurred reading the scratchpad
 * @param NASTATUS_COMPARATOR_INVALID_PARAMETER The given parameter is invalid interrupt.
 * @param NASTATUS_COMPARATOR_NOT_SUPPORTED   The comparator does not support 
 * @param NASTATUS_COMPARATOR_INVALID_ID      The given id is invalid. Unable to locate the comparator.
 * @param NASTATUS_GFX_BBUFFER_DISABLED       The backbuffer is currently disabled
 * @param NASTATUS_GFX_INVALID_DEVICE         The specified device is not valid
 * @param NASTATUS_GFX_VMODE_NOT_FOUND        A matching video mode could not be found
 * @param NASTATUS_GFX_VMODE_NOT_EXACT        Warning: The returned video mode is the closest that could be found. It does not match the requested resolution exactly.
 * @param NASTATUS_GFX_FILE_OPEN_ERROR        Unable to open the specified file
 * @param NASTATUS_GFX_FILE_READ_ERROR        Unable to read the specified file
 * @param NASTATUS_GFX_JPEG_DECODE_ERROR      Unable to decode the specified jpeg data
 * @param NASTATUS_GFX_UNSUPPORTED            The requested operation is not supported
 * @param NASTATUS_GFX_EVENT_HEAP_ERROR       An error occured creating the event heap storage
 * @param NASTATUS_GFX_EVENT_TIMEOUT          Warning: Timeout occured waiting for the next event
 * @param NASTATUS_GFX_EVENT_QUEUE_ERROR      There was an error working with the event queue
 * @param NASTATUS_GFX_EVENT_QUEUE_FULL       The event queue is full and the event could not be added
 * @param NASTATUS_GFX_EVENT_CONSUMED         Success: The event was processed by this handler and should not be propogated
 * @param NASTATUS_GFX_EVENT_NOT_CONSUMED     Success: The event may have been processed by this handler but it should be propogated to the next listener
 * @param NASTATUS_GFX_VIO_ERROR              Unable to read/write to the specified Virtual I/O source
 * @param NASTATUS_GFX_DECODE_ERROR           Unable to decode the specified image
 * @param NASTATUS_GFX_DECODE_CANCELLED       The decode request has been cancelled
 * @param NASTATUS_LCD_OPEN_ERROR             Error opening the specified lcd device
 * @param NASTATUS_LCD_SET_TYPE_ERROR         Error setting the LCD type
 * @param NASTATUS_LCD_SET_PALETTE_ERROR      Error setting the lcd pallete entry
 * @param NASTATUS_LCD_GET_BUFF_ERROR         Error retrieving the lcd framebuffer
 * @param NASTATUS_LCD_SET_CTL_ERROR          Error enabling the lcd control
 * @param NASTATUS_LCD_SET_PWR_ERROR          Error powering on the lcd
 * @param NASTATUS_LCD_SET_BUFF_ERROR         Error setting the lcd frame buffer pointer
 * @param NASTATUS_SM_INIT_ERROR              Error initializing the SM501
 * @param NASTATUS_SM_SET_VMODE_ERROR         Error setting the SM501 video mode
 * @param NASTATUS_RTC_FAIL                   Invalid config value passed
 * @param NASTATUS_RTC_NOT_INITIALIZED        RTC device not initialized
 * @param NASTATUS_RTC_NOT_AVAILABLE          Required RTC resource is not available
 * @param NASTATUS_ETH_FAILURE                Error in Ethernet driver
 * @param NASTATUS_QUAD_BAD_DEPTH             Request exceeds fifo depth
 * @param NASTATUS_QUAD_NO_FUNCTION           Requested function is not supplied
 * @param NASTATUS_SPI_MASTER_DEVICE_ACTIVE	  SPI Master is currently running
 * @param NASTATUS_SPI_MASTER_DUPLICATE_DEVICE  Device is already exist
 * @param NASTATUS_SPI_MASTER_INVALID_SETTINGS	Invalid settings
 * @param NASTATUS_SPI_MASTER_INVALID_BUFFER    Invalid buffer		
 * @param NASTATUS_SPI_MASTER_DMA_FAILED		Encountered DMA error
 * @param NASTATUS_SPI_MASTER_DEVICE_NOT_FOUND  Device is not found
 * @param NASTATUS_SPI_SLAVE_ACTIVE			    SPI Slave is currently running
 * @param NASTATUS_SPI_SLAVE_INVALID_SETTINGS	Invalid settings
 * @param NASTATUS_SPI_SLAVE_RING_FULL			Ring size is full.
 * @param NASTATUS_IOP_BAD_PROCESSOR            The processor argument is out of range 
 * @param NASTATUS_IOP_BAD_BUFFER               The buffer pointer is NULL or invalid (not 32 bit aligned)
 * @param NASTATUS_IOP_BAD_LENGTH               The length argument is invalid   
 * @param NASTATUS_IOP_BAD_CALLBACK             The callback is invalid   
 * @param NASTATUS_IOP_QUEUE_IS_FULL            The receive/transmit queue is full
 * @param NASTATUS_IOP_BAD_ARGUMENT             Invalid argument  
 * @param NASTATUS_IOP_BAD_DIVISOR              The clock divisor for IOP is invalid
 * @param NASTATUS_IOP_INUSE                    The ISR callback is already registered
 * @param NASTATUS_IOP_SYSTEM_ERROR             Failed to install ISR from naIopProcessorDownloadProgram
 * @param NASTATUS_IOP_BAD_CODE                 The interrupt code field is out of range
 * @param NASTATUS_IOP_BAD_PROGRAM              The program argument to naIopProcessorDownloadProgram is invalid
 * @param NASTATUS_IOP_UNSUPPORTED_FORMAT       The format field in the program header is not supported
 * @param NASTATUS_IOP_UNSUPPORTED_PROCESSOR    The processorType field in the program header is not supported
 * @param NASTATUS_IOP_INVALID_CONFIGURATION    The hardware assist configuration is invalid
 * @param NASTATUS_IOP_PIC_IS_RUNNING           The PIC isn't stopped before calling naIopProcessorDownloadProgram   
 * @param NASTATUS_IOP_NO_CALLBACK              No callback was installed
 * @param NASTATUS_FAT2SDIO_ALREADY_INITIALIZED     The FAT FS to SDIO driver is already initialized 
 * @param NASTATUS_FAT2SDIO_NULL_POINTER            Null pointer in function arguments
 * @param NASTATUS_FAT2SDIO_MALLOC_FAILED           Call to malloc() failed
 * @param NASTATUS_FAT2SDIO_MAX_DEVICES_EXCEEDED    The number of supported SD devices was exceeded
 * @param NASTATUS_FAT2SDIO_NOT_INITIALIZED         The FAT FS to SDIO driver is not initialized
 * @param NASTATUS_FAT2SDIO_DEVICE_NOT_FOUND        The SD device was not found
 * @param NASTATUS_FAT2SDIO_R1_STATUS_ERROR         R1 response status bits indicate that an error occurred
 * @param NASTATUS_FAT2SDIO_MUTEX_FAILED            Mutex related operation failed
 * @param NASTATUS_SDIO_BAD_INTERFACE               Invalid interface specified in function call
 * @param NASTATUS_SDIO_BAD_ARGUMENT                Invalid parameter specified in function call
 * @param NASTATUS_SDIO_BUSY                        SDIO card is busy
 * @param NASTATUS_SDIO_TIMEOUT                     PIC observed a timeout for response request from SDIO card
 * @param NASTATUS_SDIO_CARD_CRCERR                 SD card reported crc error on a block transfer to the card
 * @param NASTATUS_SDIO_ECHO_FAILURE                Failure of echo pattern on response to command 8
 * @param NASTATUS_SDIO_VOLTRNG_FAIL                Mismatch of voltage range in response to command 8
 * @param NASTATUS_SDIO_CARD_NOT_CAPABLE            Card has been found to be neither IO nor memeory capable
 * @param NASTATUS_BAD_RSP_CODE                     Command index in response is not the expected value
 * @param NASTATUS_SDIO_XDMA_TIMEOUT                Transmit DMA timeout
 * @param NASTATUS_SDIO_RDMA_TIMEOUT                Receive DMA timeout
 * @param NASTATUS_CBUS_BAD_INTERFACE               Invalid interface specified
 * @param NASTATUS_CBUS_BAD_ARGUMENT                Invalid argument to function
 * @param NASTATUS_CBUS_NO_BAUD_RATE                Baud rate for CAN bus has not been set
 * @param NASTATUS_CBUS_NOT_STOPPED                 Operation cannot be completed because the driver is running
 * @param NASTATUS_CBUS_NOT_RUNNING                 CAN controller is not running
 * @param NASTATUS_CBUS_OUT_OF_RESOURCES            Unable to allocated the resources needed to complete this call
 * @param NASTATUS_CBUS_NO_RECEIVER                 The receiver thread has not been initialized yet
 * @param NASTATUS_CBUS_RECEIVER_ALREADY_INSTALLED  CAN receiver thread has already been installed
 * @param NASTATUS_CBUS_NOT_OPEN                    You must call naCbusOpen before calling any other API
 * @param NASTATUS_CBUS_ALREADY_OPEN                The CAN driver has already been opened
 * @param NASTATUS_CBUS_OFF_BUS                     The controller is in the 'bus off' state
 * @param NASTATUS_CBUS_RTRS_ALREADY_ALLOCATED      RTR control block has already been allocated
 * @param NASTATUS_CBUS_RTRS_NOT_ALLOCATED          RTR control block has not been allocated yet 
 *
 *
 * @external
 * @category general_purpose
 * @since 6.2
 * @name NaStatusValues "NaStatus values"
 * @include "nastatus.h"
 *
 * @see @link NaStatus
 * @see @link DeprecatedNaStatusValues
*ad*/
#define NASTATUS_SUCCESS                                  (0x0)
#define NASTATUS_INVALID_CHANNEL                          (NASTATUS_ERROR   | NASTATUS_GEN   | 0x0001UL)  /* 0x80010001 */
#define NASTATUS_INVALID_HANDLE                           (NASTATUS_ERROR   | NASTATUS_GEN   | 0x0002UL)  /* 0x80010002 */
#define NASTATUS_INVALID_PARAMETER                        (NASTATUS_ERROR   | NASTATUS_GEN   | 0x0003UL)  /* 0x80010003 */
#define NASTATUS_RESOURCE_ERROR                           (NASTATUS_ERROR   | NASTATUS_GEN   | 0x0004UL)  /* 0x80010004 */
#define NASTATUS_IN_USE                                   (NASTATUS_ERROR   | NASTATUS_GEN   | 0x0005UL)  /* 0x80010005 */
#define NASTATUS_ENTRY_NOT_FOUND                          (NASTATUS_ERROR   | NASTATUS_GEN   | 0x0006UL)  /* 0x80010006 */   
#define NASTATUS_MALLOC_ERROR                             (NASTATUS_ERROR   | NASTATUS_GEN   | 0x0007UL)  /* 0x80010007 */
#define NASTATUS_NOT_INITIALIZED                          (NASTATUS_ERROR   | NASTATUS_GEN   | 0x0008UL)  /* 0x80010008 */
#define NASTATUS_PD_OUT_OF_RESOURCES                      (NASTATUS_ERROR   | NASTATUS_PD    | 0x0001UL)  /* 0x80020001 */
#define NASTATUS_PD_INVALID_TRIGGER                       (NASTATUS_ERROR   | NASTATUS_PD    | 0x0002UL)  /* 0x80020002 */
#define NASTATUS_PD_NO_TRIGGER                            (NASTATUS_ERROR   | NASTATUS_PD    | 0x0003UL)  /* 0x80020003 */
#define NASTATUS_PD_NULL_POINTER                          (NASTATUS_ERROR   | NASTATUS_PD    | 0x0004UL)  /* 0x80020004 */ 
#define NASTATUS_PD_BAD_PORT                              (NASTATUS_ERROR   | NASTATUS_PD    | 0x0005UL)  /* 0x80020005 */
#define NASTATUS_PD_BAD_DIVISOR                           (NASTATUS_ERROR   | NASTATUS_PD    | 0x0006UL)  /* 0x80020006 */ 
#define NASTATUS_PD_BAD_MODE                              (NASTATUS_ERROR   | NASTATUS_PD    | 0x0007UL)  /* 0x80020007 */
#define NASTATUS_PD_FAILED_TO_SET_INTR                    (NASTATUS_ERROR   | NASTATUS_PD    | 0x0008UL)  /* 0x80020008 */
#define NASTATUS_PD_ALREADY_IN_CATNAP                     (NASTATUS_ERROR   | NASTATUS_PD    | 0x0009UL)  /* 0x80020009 */
#define NASTATUS_PD_WATCHDOG_ERROR                        (NASTATUS_ERROR   | NASTATUS_PD    | 0x000AUL)  /* 0x8002000A */
#define NASTATUS_IPSEC_OUT_OF_RESOURCES                   (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x0001UL)  /* 0x80030001 */
#define NASTATUS_IPSEC_INVALID_PARAMETER                  (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x0002UL)  /* 0x80030002 */
#define NASTATUS_IPSEC_NOT_INITIALIZED                    (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x0003UL)  /* 0x80030003 */
#define NASTATUS_IPSEC_INVALID_PEER                       (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x0004UL)  /* 0x80030004 */
#define NASTATUS_IPSEC_INVALID_KEY                        (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x0005UL)  /* 0x80030005 */
#define NASTATUS_IPSEC_INVALID_ENCRYPTION_ALGORITHM       (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x0006UL)  /* 0x80030006 */
#define NASTATUS_IPSEC_INVALID_KEYLENGTH                  (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x0007UL)  /* 0x80030007 */
#define NASTATUS_IPSEC_INVALID_AUTHENTICATION_ALGORITHM   (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x0008UL)  /* 0x80030008 */
#define NASTATUS_IPSEC_INVALID_HANDLE                     (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x0009UL)  /* 0x80030009 */
#define NASTATUS_IPSEC_INVALID_SECONDS                    (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x000AUL)  /* 0x8003000A */
#define NASTATUS_IPSEC_INVALID_KILOBYTES                  (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x000BUL)  /* 0x8003000B */
#define NASTATUS_IPSEC_INVALID_INDEX                      (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x000CUL)  /* 0x8003000C */
#define NASTATUS_IPSEC_WRONG_LIBRARY                      (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x000DUL)  /* 0x8003000D */
#define NASTATUS_IPSEC_ALREADY_INITIALIZED                (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x000EUL)  /* 0x8003000E */
#define NASTATUS_IPSEC_INVALID_DEFAULT_POLICY             (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x000FUL)  /* 0x8003000F */
#define NASTATUS_IPSEC_INVALID_SELECTOR                   (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x0010UL)  /* 0x80030010 */
#define NASTATUS_IPSEC_INVALID_ADDRESS                    (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x0011UL)  /* 0x80030011 */
#define NASTATUS_IPSEC_INVALID_IP_ADDRESS                 (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x0012UL)  /* 0x80030012 */
#define NASTATUS_IPSEC_SA_NOT_ACTIVE                      (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x0013UL)  /* 0x80030013 */
#define NASTATUS_IPSEC_INVALID_TABLE_ENTRY                (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x0014UL)  /* 0x80030014 */
#define NASTATUS_IPSEC_INVALID_RULES                      (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x0015UL)  /* 0x80030015 */
#define NASTATUS_IPSEC_SA_IN_USE                          (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x0016UL)  /* 0x80030016 */
#define NASTATUS_IPSEC_NOT_CONFIGURED                     (NASTATUS_ERROR   | NASTATUS_IPSEC | 0x0018UL)  /* 0x80030018 */
#define NASTATUS_TCPIP_CANNOT_ADD_ROUTE                   (NASTATUS_ERROR   | NASTATUS_TCPIP | 0x0001UL)  /* 0x80040001 */ 
#define NASTATUS_TCPIP_ARP_CACHE_FULL                     (NASTATUS_ERROR   | NASTATUS_TCPIP | 0x0002UL)  /* 0x80040002 */    
#define NASTATUS_EKAHAU_NO_MEMORY                         (NASTATUS_ERROR   | NASTATUS_EKAHAU| 0x0008UL)  /* 0x80070008 */
#define NASTATUS_SNMP_INVALID_POINTER                     (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x0001UL)  /* 0x80080001 */
#define NASTATUS_SNMP_NO_MEMORY                           (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x0002UL)  /* 0x80080002 */
#define NASTATUS_SNMP_NOSUCH_TRAPADDR                     (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x0003UL)  /* 0x80080003 */
#define NASTATUS_SNMP_CANNOT_SEND_TRAP                    (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x0004UL)  /* 0x80080004 */
#define NASTATUS_SNMP_INVALID_DEST_PORT                   (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x0005UL)  /* 0x80080005 */
#define NASTATUS_SNMP_GENERAL_ERROR                       (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x0006UL)  /* 0x80080006 */
#define NASTATUS_SNMP_TRAPS_MAX_REACHED                   (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x0007UL)  /* 0x80080007 */
#define NASTATUS_SNMP_AGENT_LOCKED                        (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x0008UL)  /* 0x80080008 */
#define NASTATUS_SNMP_INVALID_SYSOID                      (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x0009UL)  /* 0x80080009 */
#define NASTATUS_SNMP_INVALID_SYSDESCR                    (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x000AUL)  /* 0x8008000A */
#define NASTATUS_SNMP_AGENT_STARTED                       (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x000BUL)  /* 0x8008000B */
#define NASTATUS_SNMP_INVALID_COMMUNITY                   (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x000CUL)  /* 0x8008000C */
#define NASTATUS_SNMP_INVALID_SYSSERV                     (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x000DUL)  /* 0x8008000D */
#define NASTATUS_SNMP_AGENT_CANNOT_START                  (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x000EUL)  /* 0x8008000E */
#define NASTATUS_SNMP_INVALID_ENTERPRISE                  (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x000FUL)  /* 0x8008000F */
#define NASTATUS_SNMP_INVALID_SYSCONTACT                  (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x0010UL)  /* 0x80080010 */
#define NASTATUS_SNMP_INVALID_SYSLOCATION                 (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x0011UL)  /* 0x80080011 */
#define NASTATUS_SNMP_INVALID_SYSNAME                     (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x0012UL)  /* 0x80080012 */
#define NASTATUS_SNMP_INVALID_ARGUMENT                    (NASTATUS_ERROR   | NASTATUS_SNMP  | 0x0013UL)  /* 0x80080013 */
#define NASTATUS_IF_INVALID_IF_NAME                       (NASTATUS_ERROR   | NASTATUS_IF    | 0x0001UL)  /* 0x80090001 */
#define NASTATUS_IF_ALREADY_CONFIGURED                    (NASTATUS_ERROR   | NASTATUS_IF    | 0x0002UL)  /* 0x80090002 */
#define NASTATUS_IF_INTERNAL_ERROR                        (NASTATUS_ERROR   | NASTATUS_IF    | 0x0003UL)  /* 0x80090003 */
#define NASTATUS_IF_INVALID_LINK_TYPE                     (NASTATUS_ERROR   | NASTATUS_IF    | 0x0004UL)  /* 0x80090004 */
#define NASTATUS_IF_INVALID_PARAMETER                     (NASTATUS_ERROR   | NASTATUS_IF    | 0x0005UL)  /* 0x80090005 */
#define NASTATUS_IF_ADDR_CONFLICT                         (NASTATUS_ERROR   | NASTATUS_IF    | 0x0007UL)  /* 0x80090007 */
#define NASTATUS_IF_NOT_CONFIGURED                        (NASTATUS_ERROR   | NASTATUS_IF    | 0x0008UL)  /* 0x80090008 */
#define NASTATUS_IF_CONFIGURATION_IN_PROGRESS             (NASTATUS_SUCCESS | NASTATUS_IF    | 0x0009UL)  /* 0x00090009 */
#define NASTATUS_AES_MUTEX_FAILED                         (NASTATUS_ERROR   | NASTATUS_AES   | 0x0001UL)  /* 0x800A0001 */
#define NASTATUS_AES_INSTALL_ISR_FAILED                   (NASTATUS_ERROR   | NASTATUS_AES   | 0x0002UL)  /* 0x800A0002 */
#define NASTATUS_AES_ALREADY_INIT                         (NASTATUS_ERROR   | NASTATUS_AES   | 0x0003UL)  /* 0x800A0003 */
#define NASTATUS_AES_NULL_POINTER                         (NASTATUS_ERROR   | NASTATUS_AES   | 0x0004UL)  /* 0x800A0004 */
#define NASTATUS_AES_INVALID_OPERATION                    (NASTATUS_ERROR   | NASTATUS_AES   | 0x0005UL)  /* 0x800A0005 */
#define NASTATUS_AES_ZERO_LENGTH_BUFFER                   (NASTATUS_ERROR   | NASTATUS_AES   | 0x0006UL)  /* 0x800A0006 */
#define NASTATUS_AES_SEMAPHORE_FAILED                     (NASTATUS_ERROR   | NASTATUS_AES   | 0x0007UL)  /* 0x800A0007 */
#define NASTATUS_AES_INVALID_DMA_DESC_INDEX               (NASTATUS_ERROR   | NASTATUS_AES   | 0x0008UL)  /* 0x800A0008 */
#define NASTATUS_AES_INVALID_KEY_SIZE                     (NASTATUS_ERROR   | NASTATUS_AES   | 0x0009UL)  /* 0x800A0009 */
#define NASTATUS_AES_NOT_INIT                             (NASTATUS_ERROR   | NASTATUS_AES   | 0x000AUL)  /* 0x800A000A */
#define NASTATUS_AES_INVALID_CCM_L_VALUE                  (NASTATUS_ERROR   | NASTATUS_AES   | 0x000BUL)  /* 0x800A000B */
#define NASTATUS_AES_INVALID_CCM_M_VALUE                  (NASTATUS_ERROR   | NASTATUS_AES   | 0x000CUL)  /* 0x800A000C */
#define NASTATUS_AES_MALLOC_FAILED                        (NASTATUS_ERROR   | NASTATUS_AES   | 0x000DUL)  /* 0x800A000D */
#define NASTATUS_AES_CBUFFER_FULL                         (NASTATUS_ERROR   | NASTATUS_AES   | 0x000EUL)  /* 0x800A000E */
#define NASTATUS_AES_OPENSSL_FUNCS_FAILED                 (NASTATUS_ERROR   | NASTATUS_AES   | 0x000FUL)  /* 0x800A000F */
#define NASTATUS_AES_INVALID_BUFFER_LENGTH                (NASTATUS_ERROR   | NASTATUS_AES   | 0x0010UL)  /* 0x800A0010 */
#define NASTATUS_AES_UNALIGNED_BUFFER                     (NASTATUS_ERROR   | NASTATUS_AES   | 0x0011UL)  /* 0x800A0011 */
#define NASTATUS_AES_MAX_XFER_SIZE_EXCEEDED               (NASTATUS_ERROR   | NASTATUS_AES   | 0x0012UL)  /* 0x800A0012 */
#define NASTATUS_A2D_BAD_ARGUMENT                         (NASTATUS_ERROR   | NASTATUS_A2D   | 0x0001UL)  /* 0x800B0001 */
#define NASTATUS_A2D_BAD_CLOCK_RATE                       (NASTATUS_ERROR   | NASTATUS_A2D   | 0x0002UL)  /* 0x800B0002 */ 
#define NASTATUS_A2D_BAD_HANDLE                           (NASTATUS_ERROR   | NASTATUS_A2D   | 0x0003UL)  /* 0x800B0003 */ 
#define NASTATUS_A2D_BAD_INPUT                            (NASTATUS_ERROR   | NASTATUS_A2D   | 0x0004UL)  /* 0x800B0004 */ 
#define NASTATUS_A2D_BAD_LENGTH                           (NASTATUS_ERROR   | NASTATUS_A2D   | 0x0005UL)  /* 0x800B0005 */
#define NASTATUS_A2D_BUFFER_Q_FULL                        (NASTATUS_ERROR   | NASTATUS_A2D   | 0x0006UL)  /* 0x800B0006 */
#define NASTATUS_A2D_CHANNEL_IN_USE                       (NASTATUS_ERROR   | NASTATUS_A2D   | 0x0007UL)  /* 0x800B0007 */
#define NASTATUS_A2D_DUPLICATE_CALLBACK                   (NASTATUS_ERROR   | NASTATUS_A2D   | 0x0008UL)  /* 0x800B0008 */
#define NASTATUS_A2D_DUPLICATE_ID                         (NASTATUS_ERROR   | NASTATUS_A2D   | 0x0009UL)  /* 0x800B0009 */
#define NASTATUS_A2D_INVALID_STATE                        (NASTATUS_ERROR   | NASTATUS_A2D   | 0x000AUL)  /* 0x800B000a */
#define NASTATUS_A2D_ISR_ALREADY_INSTALLED                (NASTATUS_ERROR   | NASTATUS_A2D   | 0x000BUL)  /* 0x800B000b */
#define NASTATUS_A2D_ISR_NOT_INSTALLED                    (NASTATUS_ERROR   | NASTATUS_A2D   | 0x000CUL)  /* 0x800B000c */
#define NASTATUS_A2D_NO_BUFFERS                           (NASTATUS_ERROR   | NASTATUS_A2D   | 0x000DUL)  /* 0x800B000d */
#define NASTATUS_A2D_NO_SUCH_CALLBACK                     (NASTATUS_ERROR   | NASTATUS_A2D   | 0x000EUL)  /* 0x800B000e */
#define NASTATUS_A2D_NOT_AVAILABLE                        (NASTATUS_ERROR   | NASTATUS_A2D   | 0x000FUL)  /* 0x800B000f */
#define NASTATUS_A2D_NOT_CONFIGURED                       (NASTATUS_ERROR   | NASTATUS_A2D   | 0x0010UL)  /* 0x800B0010 */
#define NASTATUS_A2D_NOT_RUNNING                          (NASTATUS_ERROR   | NASTATUS_A2D   | 0x0011UL)  /* 0x800B0011 */
#define NASTATUS_A2D_NOT_SUPPORTED                        (NASTATUS_ERROR   | NASTATUS_A2D   | 0x0012UL)  /* 0x800B0012 */
#define NASTATUS_A2D_OUT_OF_RESOURCES                     (NASTATUS_ERROR   | NASTATUS_A2D   | 0x0013UL)  /* 0x800B0013 */
#define NASTATUS_A2D_UNKNOWN_IDENTIFIER                   (NASTATUS_ERROR   | NASTATUS_A2D   | 0x0014UL)  /* 0x800B0014 */
#define NASTATUS_A2D_INCORRECT_VERSION                    (NASTATUS_ERROR   | NASTATUS_A2D   | 0x0015UL)  /* 0x800B0015 */
#define NASTATUS_SCRATCHPAD_BAD_ARGUMENT                  (NASTATUS_ERROR   | NASTATUS_SCRP  | 0x0001UL)  /* 0x800C0001 */
#define NASTATUS_SCRATCHPAD_BAD_SIZE                      (NASTATUS_ERROR   | NASTATUS_SCRP  | 0x0002UL)  /* 0x800C0002 */
#define NASTATUS_SCRATCHPAD_DUPLICATE_LOCATION            (NASTATUS_ERROR   | NASTATUS_SCRP  | 0x0003UL)  /* 0x800C0003 */
#define NASTATUS_SCRATCHPAD_ARRAY_TOO_SMALL               (NASTATUS_ERROR   | NASTATUS_SCRP  | 0x0004UL)  /* 0x800C0004 */
#define NASTATUS_SCRATCHPAD_UNKNOWN                       (NASTATUS_ERROR   | NASTATUS_SCRP  | 0x0005UL)  /* 0x800C0005 */
#define NASTATUS_SCRATCHPAD_INVALID_HANDLE                (NASTATUS_ERROR   | NASTATUS_SCRP  | 0x0006UL)  /* 0x800C0006 */
#define NASTATUS_SCRATCHPAD_OUT_OF_BOUNDS                 (NASTATUS_ERROR   | NASTATUS_SCRP  | 0x0007UL)  /* 0x800C0007 */
#define NASTATUS_SCRATCHPAD_FAILURE                       (NASTATUS_ERROR   | NASTATUS_SCRP  | 0x0008UL)  /* 0x800C0008 */
#define NASTATUS_COMPARATOR_INVALID_PARAMETER             (NASTATUS_ERROR   | NASTATUS_CMPR  | 0x0001UL)  /* 0x800E0001 */
#define NASTATUS_COMPARATOR_NOT_SUPPORTED                 (NASTATUS_ERROR   | NASTATUS_CMPR  | 0x0002UL)  /* 0x800E0002 */
#define NASTATUS_COMPARATOR_INVALID_ID                    (NASTATUS_ERROR   | NASTATUS_CMPR  | 0x0003UL)  /* 0x800E0003 */
#define NASTATUS_GFX_BBUFFER_DISABLED                     (NASTATUS_ERROR   | NASTATUS_GFX   | 0x0001UL)  /* 0x80100001 */
#define NASTATUS_GFX_INVALID_DEVICE                       (NASTATUS_ERROR   | NASTATUS_GFX   | 0x0002UL)  /* 0x80100002 */
#define NASTATUS_GFX_VMODE_NOT_FOUND                      (NASTATUS_ERROR   | NASTATUS_GFX   | 0x0003UL)  /* 0x80100003 */
#define NASTATUS_GFX_VMODE_NOT_EXACT                      (NASTATUS_WARNING | NASTATUS_GFX   | 0x0004UL)  /* 0x40100004 */
#define NASTATUS_GFX_FILE_OPEN_ERROR                      (NASTATUS_ERROR   | NASTATUS_GFX   | 0x0005UL)  /* 0x80100005 */
#define NASTATUS_GFX_FILE_READ_ERROR                      (NASTATUS_ERROR   | NASTATUS_GFX   | 0x0006UL)  /* 0x80100006 */
#define NASTATUS_GFX_JPEG_DECODE_ERROR                    (NASTATUS_ERROR   | NASTATUS_GFX   | 0x0007UL)  /* 0x80100007 */
#define NASTATUS_GFX_UNSUPPORTED                          (NASTATUS_ERROR   | NASTATUS_GFX   | 0x0008UL)  /* 0x80100008 */
#define NASTATUS_GFX_EVENT_HEAP_ERROR                     (NASTATUS_ERROR   | NASTATUS_GFX   | 0x0009UL)  /* 0x80100009 */
#define NASTATUS_GFX_EVENT_TIMEOUT                        (NASTATUS_WARNING | NASTATUS_GFX   | 0x000AUL)  /* 0x4010000A */
#define NASTATUS_GFX_EVENT_QUEUE_ERROR                    (NASTATUS_ERROR   | NASTATUS_GFX   | 0x000BUL)  /* 0x8010000B */
#define NASTATUS_GFX_EVENT_QUEUE_FULL                     (NASTATUS_ERROR   | NASTATUS_GFX   | 0x000CUL)  /* 0x8010000C */
#define NASTATUS_GFX_EVENT_CONSUMED                       (NASTATUS_SUCCESS | NASTATUS_GFX   | 0x000DUL)  /* 0x0010000D */
#define NASTATUS_GFX_EVENT_NOT_CONSUMED                   (NASTATUS_SUCCESS | NASTATUS_GFX   | 0x000EUL)  /* 0x0010000E */
#define NASTATUS_GFX_VIO_ERROR                            (NASTATUS_ERROR   | NASTATUS_GFX   | 0x000FUL)  /* 0x8010000F */
#define NASTATUS_GFX_DECODE_ERROR                         (NASTATUS_ERROR   | NASTATUS_GFX   | 0x0010UL)  /* 0x80100010 */
#define NASTATUS_GFX_DECODE_CANCELLED                     (NASTATUS_ERROR   | NASTATUS_GFX   | 0x0011UL)  /* 0x80100011 */
#define NASTATUS_LCD_OPEN_ERROR                           (NASTATUS_ERROR   | NASTATUS_LCD   | 0x0001UL)  /* 0x80110001 */
#define NASTATUS_LCD_SET_TYPE_ERROR                       (NASTATUS_ERROR   | NASTATUS_LCD   | 0x0002UL)  /* 0x80110002 */
#define NASTATUS_LCD_GET_BUFF_ERROR                       (NASTATUS_ERROR   | NASTATUS_LCD   | 0x0003UL)  /* 0x80110003 */
#define NASTATUS_LCD_SET_CTL_ERROR                        (NASTATUS_ERROR   | NASTATUS_LCD   | 0x0004UL)  /* 0x80110004 */
#define NASTATUS_LCD_SET_PWR_ERROR                        (NASTATUS_ERROR   | NASTATUS_LCD   | 0x0005UL)  /* 0x80110005 */
#define NASTATUS_LCD_SET_BUFF_ERROR                       (NASTATUS_ERROR   | NASTATUS_LCD   | 0x0006UL)  /* 0x80110006 */
#define NASTATUS_LCD_SET_PALETTE_ERROR                    (NASTATUS_ERROR   | NASTATUS_LCD   | 0x0002UL)  /* 0x80110007 */
#define NASTATUS_SM_INIT_ERROR                            (NASTATUS_ERROR   | NASTATUS_SM    | 0x0001UL)  /* 0x80120001 */
#define NASTATUS_SM_SET_VMODE_ERROR                       (NASTATUS_ERROR   | NASTATUS_SM    | 0x0002UL)  /* 0x80120002 */
#define NASTATUS_RTC_FAIL                                 (NASTATUS_ERROR   | NASTATUS_RTC   | 0x0001UL)  /* 0x80130001 */
#define NASTATUS_RTC_NOT_INITIALIZED                      (NASTATUS_ERROR   | NASTATUS_RTC   | 0x0002UL)  /* 0x80130002 */
#define NASTATUS_RTC_NOT_AVAILABLE                        (NASTATUS_ERROR   | NASTATUS_RTC   | 0x0003UL)  /* 0x80130003 */
#define NASTATUS_ETH_FAILURE                              (NASTATUS_ERROR   | NASTATUS_ETH   | 0x0001UL)  /* 0x80140001 */
#define NASTATUS_QUAD_BAD_DEPTH                           (NASTATUS_ERROR   | NASTATUS_QUAD  | 0x0001UL)  /* 0x80150001 */
#define NASTATUS_QUAD_NO_FUNCTION                         (NASTATUS_ERROR   | NASTATUS_QUAD  | 0x0002UL)  /* 0x80150002 */
#define NASTATUS_SPI_MASTER_DEVICE_ACTIVE			      (NASTATUS_WARNING | NASTATUS_SPI_MASTER  | 0x0001UL)  /* 0x80160001 */  
#define NASTATUS_SPI_MASTER_DUPLICATE_DEVICE              (NASTATUS_ERROR   | NASTATUS_SPI_MASTER  | 0x0002UL)  /* 0x80160002 */
#define NASTATUS_SPI_MASTER_INVALID_SETTINGS	          (NASTATUS_ERROR   | NASTATUS_SPI_MASTER  | 0x0003UL)  /* 0x80160003 */
#define NASTATUS_SPI_MASTER_INVALID_BUFFER		          (NASTATUS_ERROR   | NASTATUS_SPI_MASTER  | 0x0004UL)  /* 0x80160004 */
#define NASTATUS_SPI_MASTER_DMA_FAILED			          (NASTATUS_ERROR   | NASTATUS_SPI_MASTER  | 0x0005UL)  /* 0x80160005 */
#define NASTATUS_SPI_MASTER_DEVICE_NOT_FOUND              (NASTATUS_ERROR   | NASTATUS_SPI_MASTER  | 0x0006UL)  /* 0x80160006 */
#define NASTATUS_SPI_SLAVE_ACTIVE			              (NASTATUS_WARNING | NASTATUS_SPI_SLAVE   | 0x0001UL)  /* 0x80170001 */  
#define NASTATUS_SPI_SLAVE_INVALID_SETTINGS	              (NASTATUS_ERROR   | NASTATUS_SPI_SLAVE   | 0x0002UL)  /* 0x80170002 */
#define NASTATUS_SPI_SLAVE_RING_FULL			          (NASTATUS_ERROR   | NASTATUS_SPI_SLAVE   | 0x0003UL)  /* 0x80170003 */
#define NASTATUS_IOP_BAD_PROCESSOR                        (NASTATUS_ERROR   | NASTATUS_IOP   | 0x0001UL)  /* 0x80180001 */ 
#define NASTATUS_IOP_BAD_BUFFER                           (NASTATUS_ERROR   | NASTATUS_IOP   | 0x0002UL)  /* 0x80180002 */ 
#define NASTATUS_IOP_BAD_LENGTH                           (NASTATUS_ERROR   | NASTATUS_IOP   | 0x0003UL)  /* 0x80180003 */ 
#define NASTATUS_IOP_BAD_CALLBACK                         (NASTATUS_ERROR   | NASTATUS_IOP   | 0x0004UL)  /* 0x80180004 */ 
#define NASTATUS_IOP_QUEUE_IS_FULL                        (NASTATUS_ERROR   | NASTATUS_IOP   | 0x0005UL)  /* 0x80180005 */ 
#define NASTATUS_IOP_BAD_ARGUMENT                         (NASTATUS_ERROR   | NASTATUS_IOP   | 0x0006UL)  /* 0x80180006 */ 
#define NASTATUS_IOP_BAD_DIVISOR                          (NASTATUS_ERROR   | NASTATUS_IOP   | 0x0007UL)  /* 0x80180007 */ 
#define NASTATUS_IOP_INUSE                                (NASTATUS_ERROR   | NASTATUS_IOP   | 0x0008UL)  /* 0x80180008 */ 
#define NASTATUS_IOP_SYSTEM_ERROR                         (NASTATUS_ERROR   | NASTATUS_IOP   | 0x0009UL)  /* 0x80180009 */ 
#define NASTATUS_IOP_BAD_CODE                             (NASTATUS_ERROR   | NASTATUS_IOP   | 0x000AUL)  /* 0x8018000A */ 
#define NASTATUS_IOP_BAD_PROGRAM                          (NASTATUS_ERROR   | NASTATUS_IOP   | 0x000BUL)  /* 0x8018000B */ 
#define NASTATUS_IOP_UNSUPPORTED_FORMAT                   (NASTATUS_ERROR   | NASTATUS_IOP   | 0x000CUL)  /* 0x8018000C */ 
#define NASTATUS_IOP_UNSUPPORTED_PROCESSOR                (NASTATUS_ERROR   | NASTATUS_IOP   | 0x000DUL)  /* 0x8018000D */ 
#define NASTATUS_IOP_INVALID_CONFIGURATION                (NASTATUS_ERROR   | NASTATUS_IOP   | 0x000EUL)  /* 0x8018000E */ 
#define NASTATUS_IOP_PIC_IS_RUNNING                       (NASTATUS_ERROR   | NASTATUS_IOP   | 0x000FUL)  /* 0x8018000F */ 
#define NASTATUS_IOP_NO_CALLBACK                          (NASTATUS_ERROR   | NASTATUS_IOP   | 0x0010UL)  /* 0x80180010 */ 
#define NASTATUS_FAT2SDIO_ALREADY_INITIALIZED             (NASTATUS_ERROR   | NASTATUS_FAT2SDIO   | 0x0001UL)  /* 0x80190001 */ 
#define NASTATUS_FAT2SDIO_NULL_POINTER                    (NASTATUS_ERROR   | NASTATUS_FAT2SDIO   | 0x0002UL)  /* 0x80190002 */ 
#define NASTATUS_FAT2SDIO_MALLOC_FAILED                   (NASTATUS_ERROR   | NASTATUS_FAT2SDIO   | 0x0003UL)  /* 0x80190003 */ 
#define NASTATUS_FAT2SDIO_MAX_DEVICES_EXCEEDED            (NASTATUS_ERROR   | NASTATUS_FAT2SDIO   | 0x0004UL)  /* 0x80190004 */ 
#define NASTATUS_FAT2SDIO_NOT_INITIALIZED                 (NASTATUS_ERROR   | NASTATUS_FAT2SDIO   | 0x0005UL)  /* 0x80190005 */ 
#define NASTATUS_FAT2SDIO_DEVICE_NOT_FOUND                (NASTATUS_ERROR   | NASTATUS_FAT2SDIO   | 0x0006UL)  /* 0x80190006 */ 
#define NASTATUS_FAT2SDIO_R1_STATUS_ERROR                 (NASTATUS_ERROR   | NASTATUS_FAT2SDIO   | 0x0007UL)  /* 0x80190007 */ 
#define NASTATUS_FAT2SDIO_MUTEX_FAILED                    (NASTATUS_ERROR   | NASTATUS_FAT2SDIO   | 0x0008UL)  /* 0x80190008 */ 
#define NASTATUS_SDIO_BAD_INTERFACE                       (NASTATUS_ERROR   | NASTATUS_SDIODRVR | 0x0001UL)  /* 0x80200001 */ 
#define NASTATUS_SDIO_BAD_ARGUMENT                        (NASTATUS_ERROR   | NASTATUS_SDIODRVR | 0x0002UL)  /* 0x80200002 */ 
#define NASTATUS_SDIO_BUSY                                (NASTATUS_ERROR   | NASTATUS_SDIODRVR | 0x0003UL)  /* 0x80200003 */ 
#define NASTATUS_SDIO_TIMEOUT                             (NASTATUS_ERROR   | NASTATUS_SDIODRVR | 0x0004UL)  /* 0x80200004 */ 
#define NASTATUS_SDIO_CARD_CRCERR                         (NASTATUS_ERROR   | NASTATUS_SDIODRVR | 0x0005UL)  /* 0x80200005 */ 
#define NASTATUS_SDIO_ECHO_FAILURE                        (NASTATUS_ERROR   | NASTATUS_SDIODRVR | 0x0006UL)  /* 0x80200006 */ 
#define NASTATUS_SDIO_VOLTRNG_FAIL                        (NASTATUS_ERROR   | NASTATUS_SDIODRVR | 0x0007UL)  /* 0x80200007 */ 
#define NASTATUS_SDIO_CARD_NOT_CAPABLE                    (NASTATUS_ERROR   | NASTATUS_SDIODRVR | 0x0008UL)  /* 0x80200008 */ 
#define NASTATUS_BAD_RSP_CODE                             (NASTATUS_ERROR   | NASTATUS_SDIODRVR | 0x0009UL)  /* 0x80200009 */ 
#define NASTATUS_SDIO_XDMA_TIMEOUT                        (NASTATUS_ERROR   | NASTATUS_SDIODRVR | 0x000aUL)  /* 0x8020000a */ 
#define NASTATUS_SDIO_RDMA_TIMEOUT                        (NASTATUS_ERROR   | NASTATUS_SDIODRVR | 0x000bUL)  /* 0x8020000b */ 
#define NASTATUS_CBUS_BAD_INTERFACE                       (NASTATUS_ERROR   | NASTATUS_CBUS | 0x0001UL)  /* 0x801a0001 */ 
#define NASTATUS_CBUS_BAD_ARGUMENT                        (NASTATUS_ERROR   | NASTATUS_CBUS | 0x0002UL)  /* 0x801a0002 */ 
#define NASTATUS_CBUS_NO_BAUD_RATE                        (NASTATUS_ERROR   | NASTATUS_CBUS | 0x0003UL)  /* 0x801a0003 */ 
#define NASTATUS_CBUS_NOT_STOPPED                         (NASTATUS_ERROR   | NASTATUS_CBUS | 0x0004UL)  /* 0x801a0004 */ 
#define NASTATUS_CBUS_NOT_RUNNING                         (NASTATUS_ERROR   | NASTATUS_CBUS | 0x0005UL)  /* 0x801a0005 */ 
#define NASTATUS_CBUS_OUT_OF_RESOURCES                    (NASTATUS_ERROR   | NASTATUS_CBUS | 0x0006UL)  /* 0x801a0006 */ 
#define NASTATUS_CBUS_NO_RECEIVER                         (NASTATUS_ERROR   | NASTATUS_CBUS | 0x0007UL)  /* 0x801a0007 */ 
#define NASTATUS_CBUS_RECEIVER_ALREADY_INSTALLED          (NASTATUS_ERROR   | NASTATUS_CBUS | 0x0008UL)  /* 0x801a0008 */ 
#define NASTATUS_CBUS_NOT_OPEN                            (NASTATUS_ERROR   | NASTATUS_CBUS | 0x0009UL)  /* 0x801a0009 */ 
#define NASTATUS_CBUS_ALREADY_OPEN                        (NASTATUS_ERROR   | NASTATUS_CBUS | 0x000AUL)  /* 0x801a000A */ 
#define NASTATUS_CBUS_OFF_BUS                             (NASTATUS_ERROR   | NASTATUS_CBUS | 0x000BUL)  /* 0x801a000B */ 
#define NASTATUS_CBUS_RTRS_ALREADY_ALLOCATED              (NASTATUS_ERROR   | NASTATUS_CBUS | 0x000CUL)  /* 0x801a000C */ 
#define NASTATUS_CBUS_RTRS_NOT_ALLOCATED                  (NASTATUS_ERROR   | NASTATUS_CBUS | 0x000DUL)  /* 0x801a000D */ 


/*ad*
 * 
 *                     DEPRECATED NET OS ERROR CODES                                                           
 *                                                                                                             
 * In an effort to improve consistency, all status constants are now prefixed with NASTATUS_xxx. The constants 
 * below have been mapped to the newly named constant. You may continue to refer to these old error code names 
 * in this release however they may be removed in future releases.                                             
 *
 * @param NA_SUCCESS                                  Deprecated constant equal to NASTATUS_SUCCESS
 * @param NA_STATUS_INVALID_CHANNEL                   Deprecated constant equal to NASTATUS_INVALID_CHANNEL    
 * @param NA_STATUS_INVALID_HANDLE                    Deprecated constant equal to NASTATUS_INVALID_HANDLE     
 * @param NA_STATUS_INVALID_PARAMETER                 Deprecated constant equal to NASTATUS_INVALID_PARAMETER  
 * @param NA_STATUS_RESOURCE_ERROR                    Deprecated constant equal to NASTATUS_RESOURCE_ERROR     
 * @param NA_STATUS_IN_USE                            Deprecated constant equal to NASTATUS_IN_USE             
 * @param NA_STATUS_ENTRY_NOT_FOUND                   Deprecated constant equal to NASTATUS_ENTRY_NOT_FOUND      
 * @param NA_PD_OUT_OF_RESOURCES                      Deprecated constant equal to NASTATUS_PD_OUT_OF_RESOURCES                     
 * @param NA_PD_INVALID_TRIGGER                       Deprecated constant equal to NASTATUS_PD_INVALID_TRIGGER                      
 * @param NA_PD_NO_TRIGGER                            Deprecated constant equal to NASTATUS_PD_NO_TRIGGER                           
 * @param NA_PD_NULL_POINTER                          Deprecated constant equal to NASTATUS_PD_NULL_POINTER                         
 * @param NA_PD_BAD_PORT                              Deprecated constant equal to NASTATUS_PD_BAD_PORT                             
 * @param NA_PD_BAD_DIVISOR                           Deprecated constant equal to NASTATUS_PD_BAD_DIVISOR                          
 * @param NA_PD_BAD_MODE                              Deprecated constant equal to NASTATUS_PD_BAD_MODE                             
 * @param NA_PD_FAILED_TO_SET_INTR                    Deprecated constant equal to NASTATUS_PD_FAILED_TO_SET_INTR                   
 * @param NA_PD_ALREADY_IN_CATNAP                     Deprecated constant equal to NASTATUS_PD_ALREADY_IN_CATNAP                    
 * @param NA_PD_WATCHDOG_ERROR                        Deprecated constant equal to NASTATUS_PD_WATCHDOG_ERROR                     
 * @param NA_IPSEC_OUT_OF_RESOURCES                   Deprecated constant equal to NASTATUS_IPSEC_OUT_OF_RESOURCES                  
 * @param NA_IPSEC_INVALID_PARAMETER                  Deprecated constant equal to NASTATUS_IPSEC_INVALID_PARAMETER                 
 * @param NA_IPSEC_NOT_INITIALIZED                    Deprecated constant equal to NASTATUS_IPSEC_NOT_INITIALIZED                   
 * @param NA_IPSEC_INVALID_PEER                       Deprecated constant equal to NASTATUS_IPSEC_INVALID_PEER                      
 * @param NA_IPSEC_INVALID_KEY                        Deprecated constant equal to NASTATUS_IPSEC_INVALID_KEY                       
 * @param NA_IPSEC_INVALID_ENCRYPTION_ALGORITHM       Deprecated constant equal to NASTATUS_IPSEC_INVALID_ENCRYPTION_ALGORITHM      
 * @param NA_IPSEC_INVALID_KEYLENGTH                  Deprecated constant equal to NASTATUS_IPSEC_INVALID_KEYLENGTH                 
 * @param NA_IPSEC_INVALID_AUTHENTICATION_ALGORITHM   Deprecated constant equal to NASTATUS_IPSEC_INVALID_AUTHENTICATION_ALGORITHM  
 * @param NA_IPSEC_INVALID_HANDLE                     Deprecated constant equal to NASTATUS_IPSEC_INVALID_HANDLE                    
 * @param NA_IPSEC_INVALID_SECONDS                    Deprecated constant equal to NASTATUS_IPSEC_INVALID_SECONDS                   
 * @param NA_IPSEC_INVALID_KILOBYTES                  Deprecated constant equal to NASTATUS_IPSEC_INVALID_KILOBYTES                 
 * @param NA_IPSEC_INVALID_INDEX                      Deprecated constant equal to NASTATUS_IPSEC_INVALID_INDEX                     
 * @param NA_IPSEC_WRONG_LIBRARY                      Deprecated constant equal to NASTATUS_IPSEC_WRONG_LIBRARY                     
 * @param NA_IPSEC_ALREADY_INITIALIZED                Deprecated constant equal to NASTATUS_IPSEC_ALREADY_INITIALIZED               
 * @param NA_IPSEC_INVALID_DEFAULT_POLICY             Deprecated constant equal to NASTATUS_IPSEC_INVALID_DEFAULT_POLICY            
 * @param NA_IPSEC_INVALID_SELECTOR                   Deprecated constant equal to NASTATUS_IPSEC_INVALID_SELECTOR                  
 * @param NA_IPSEC_INVALID_ADDRESS                    Deprecated constant equal to NASTATUS_IPSEC_INVALID_ADDRESS                   
 * @param NA_IPSEC_INVALID_IP_ADDRESS                 Deprecated constant equal to NASTATUS_IPSEC_INVALID_IP_ADDRESS                
 * @param NA_IPSEC_SA_NOT_ACTIVE                      Deprecated constant equal to NASTATUS_IPSEC_SA_NOT_ACTIVE                     
 * @param NA_IPSEC_INVALID_TABLE_ENTRY                Deprecated constant equal to NASTATUS_IPSEC_INVALID_TABLE_ENTRY               
 * @param NA_IPSEC_INVALID_RULES                      Deprecated constant equal to NASTATUS_IPSEC_INVALID_RULES                     
 * @param NA_IPSEC_SA_IN_USE                          Deprecated constant equal to NASTATUS_IPSEC_SA_IN_USE                         
 * @param NA_IPSEC_NOT_CONFIGURED                     Deprecated constant equal to NASTATUS_IPSEC_NOT_CONFIGURED                    
 * @param NA_TCPIP_CANNOT_ADD_ROUTE                   Deprecated constant equal to NASTATUS_TCPIP_CANNOT_ADD_ROUTE                  
 * @param NA_TCPIP_ARP_CACHE_FULL                     Deprecated constant equal to NASTATUS_TCPIP_ARP_CACHE_FULL                    
 * @param NA_EKAHAU_NO_MEMORY                         Deprecated constant equal to NASTATUS_EKAHAU_NO_MEMORY                        
 * @param NA_SNMP_INVALID_POINTER                     Deprecated constant equal to NASTATUS_SNMP_INVALID_POINTER                    
 * @param NA_SNMP_NO_MEMORY                           Deprecated constant equal to NASTATUS_SNMP_NO_MEMORY                          
 * @param NA_SNMP_NOSUCH_TRAPADDR                     Deprecated constant equal to NASTATUS_SNMP_NOSUCH_TRAPADDR                    
 * @param NA_SNMP_CANNOT_SEND_TRAP                    Deprecated constant equal to NASTATUS_SNMP_CANNOT_SEND_TRAP                   
 * @param NA_SNMP_INVALID_DEST_PORT                   Deprecated constant equal to NASTATUS_SNMP_INVALID_DEST_PORT                  
 * @param NA_SNMP_GENERAL_ERROR                       Deprecated constant equal to NASTATUS_SNMP_GENERAL_ERROR                      
 * @param NA_SNMP_TRAPS_MAX_REACHED                   Deprecated constant equal to NASTATUS_SNMP_TRAPS_MAX_REACHED                  
 * @param NA_SNMP_AGENT_LOCKED                        Deprecated constant equal to NASTATUS_SNMP_AGENT_LOCKED                       
 * @param NA_SNMP_INVALID_SYSOID                      Deprecated constant equal to NASTATUS_SNMP_INVALID_SYSOID                     
 * @param NA_SNMP_INVALID_SYSDESCR                    Deprecated constant equal to NASTATUS_SNMP_INVALID_SYSDESCR                   
 * @param NA_SNMP_AGENT_STARTED                       Deprecated constant equal to NASTATUS_SNMP_AGENT_STARTED                      
 * @param NA_SNMP_INVALID_COMMUNITY                   Deprecated constant equal to NASTATUS_SNMP_INVALID_COMMUNITY                  
 * @param NA_SNMP_INVALID_SYSSERV                     Deprecated constant equal to NASTATUS_SNMP_INVALID_SYSSERV                    
 * @param NA_SNMP_AGENT_CANNOT_START                  Deprecated constant equal to NASTATUS_SNMP_AGENT_CANNOT_START                 
 * @param NA_SNMP_INVALID_ENTERPRISE                  Deprecated constant equal to NASTATUS_SNMP_INVALID_ENTERPRISE                 
 * @param NA_SNMP_INVALID_SYSCONTACT                  Deprecated constant equal to NASTATUS_SNMP_INVALID_SYSCONTACT                 
 * @param NA_SNMP_INVALID_SYSLOCATION                 Deprecated constant equal to NASTATUS_SNMP_INVALID_SYSLOCATION                
 * @param NA_SNMP_INVALID_SYSNAME                     Deprecated constant equal to NASTATUS_SNMP_INVALID_SYSNAME                    
 * @param NA_SNMP_INVALID_ARGUMENT                    Deprecated constant equal to NASTATUS_SNMP_INVALID_ARGUMENT                   
 * @param NA_IF_INVALID_IF_NAME                       Deprecated constant equal to NASTATUS_IF_INVALID_IF_NAME                      
 * @param NA_IF_ALREADY_CONFIGURED                    Deprecated constant equal to NASTATUS_IF_ALREADY_CONFIGURED                   
 * @param NA_IF_INTERNAL_ERROR                        Deprecated constant equal to NASTATUS_IF_INTERNAL_ERROR                       
 * @param NA_IF_INVALID_LINK_TYPE                     Deprecated constant equal to NASTATUS_IF_INVALID_LINK_TYPE                    
 * @param NA_IF_INVALID_PARAMETER                     Deprecated constant equal to NASTATUS_IF_INVALID_PARAMETER                    
 * @param NA_IF_ADDR_CONFLICT                         Deprecated constant equal to NASTATUS_IF_ADDR_CONFLICT                        
 * @param NA_IF_NOT_CONFIGURED                        Deprecated constant equal to NASTATUS_IF_NOT_CONFIGURED                       
 * @param NA_IF_CONFIGURATION_IN_PROGRESS             Deprecated constant equal to NASTATUS_IF_CONFIGURATION_IN_PROGRESS            
 * @param NA_AES_MUTEX_FAILED                         Deprecated constant equal to NASTATUS_AES_MUTEX_FAILED                        
 * @param NA_AES_INSTALL_ISR_FAILED                   Deprecated constant equal to NASTATUS_AES_INSTALL_ISR_FAILED                  
 * @param NA_AES_ALREADY_INIT                         Deprecated constant equal to NASTATUS_AES_ALREADY_INIT                        
 * @param NA_AES_NULL_POINTER                         Deprecated constant equal to NASTATUS_AES_NULL_POINTER                        
 * @param NA_AES_INVALID_OPERATION                    Deprecated constant equal to NASTATUS_AES_INVALID_OPERATION                   
 * @param NA_AES_ZERO_LENGTH_BUFFER                   Deprecated constant equal to NASTATUS_AES_ZERO_LENGTH_BUFFER                  
 * @param NA_AES_SEMAPHORE_FAILED                     Deprecated constant equal to NASTATUS_AES_SEMAPHORE_FAILED                    
 * @param NA_AES_INVALID_DMA_DESC_INDEX               Deprecated constant equal to NASTATUS_AES_INVALID_DMA_DESC_INDEX              
 * @param NA_AES_INVALID_KEY_SIZE                     Deprecated constant equal to NASTATUS_AES_INVALID_KEY_SIZE                    
 * @param NA_AES_NOT_INIT                             Deprecated constant equal to NASTATUS_AES_NOT_INIT                            
 * @param NA_AES_INVALID_CCM_L_VALUE                  Deprecated constant equal to NASTATUS_AES_INVALID_CCM_L_VALUE                 
 * @param NA_AES_INVALID_CCM_M_VALUE                  Deprecated constant equal to NASTATUS_AES_INVALID_CCM_M_VALUE                 
 * @param NA_AES_MALLOC_FAILED                        Deprecated constant equal to NASTATUS_AES_MALLOC_FAILED                       
 * @param NA_AES_CBUFFER_FULL                         Deprecated constant equal to NASTATUS_AES_CBUFFER_FULL                        
 * @param NA_AES_OPENSSL_FUNCS_FAILED                 Deprecated constant equal to NASTATUS_AES_OPENSSL_FUNCS_FAILED                
 * @param NA_AES_INVALID_BUFFER_LENGTH                Deprecated constant equal to NASTATUS_AES_INVALID_BUFFER_LENGTH               
 * @param NA_AES_UNALIGNED_BUFFER                     Deprecated constant equal to NASTATUS_AES_UNALIGNED_BUFFER                    
 * @param NA_AES_MAX_XFER_SIZE_EXCEEDED               Deprecated constant equal to NASTATUS_AES_MAX_XFER_SIZE_EXCEEDED              
 * @param NA_A2D_BAD_ARGUMENT                         Deprecated constant equal to NASTATUS_A2D_BAD_ARGUMENT                        
 * @param NA_A2D_BAD_CLOCK_RATE                       Deprecated constant equal to NASTATUS_A2D_BAD_CLOCK_RATE                      
 * @param NA_A2D_BAD_HANDLE                           Deprecated constant equal to NASTATUS_A2D_BAD_HANDLE                          
 * @param NA_A2D_BAD_INPUT                            Deprecated constant equal to NASTATUS_A2D_BAD_INPUT                           
 * @param NA_A2D_BAD_LENGTH                           Deprecated constant equal to NASTATUS_A2D_BAD_LENGTH                          
 * @param NA_A2D_BUFFER_Q_FULL                        Deprecated constant equal to NASTATUS_A2D_BUFFER_Q_FULL                       
 * @param NA_A2D_CHANNEL_IN_USE                       Deprecated constant equal to NASTATUS_A2D_CHANNEL_IN_USE                      
 * @param NA_A2D_DUPLICATE_CALLBACK                   Deprecated constant equal to NASTATUS_A2D_DUPLICATE_CALLBACK                  
 * @param NA_A2D_DUPLICATE_ID                         Deprecated constant equal to NASTATUS_A2D_DUPLICATE_ID                        
 * @param NA_A2D_INVALID_STATE                        Deprecated constant equal to NASTATUS_A2D_INVALID_STATE                       
 * @param NA_A2D_ISR_ALREADY_INSTALLED                Deprecated constant equal to NASTATUS_A2D_ISR_ALREADY_INSTALLED               
 * @param NA_A2D_ISR_NOT_INSTALLED                    Deprecated constant equal to NASTATUS_A2D_ISR_NOT_INSTALLED                   
 * @param NA_A2D_NO_BUFFERS                           Deprecated constant equal to NASTATUS_A2D_NO_BUFFERS                          
 * @param NA_A2D_NO_SUCH_CALLBACK                     Deprecated constant equal to NASTATUS_A2D_NO_SUCH_CALLBACK                    
 * @param NA_A2D_NOT_AVAILABLE                        Deprecated constant equal to NASTATUS_A2D_NOT_AVAILABLE                       
 * @param NA_A2D_NOT_CONFIGURED                       Deprecated constant equal to NASTATUS_A2D_NOT_CONFIGURED                      
 * @param NA_A2D_NOT_RUNNING                          Deprecated constant equal to NASTATUS_A2D_NOT_RUNNING                         
 * @param NA_A2D_NOT_SUPPORTED                        Deprecated constant equal to NASTATUS_A2D_NOT_SUPPORTED                       
 * @param NA_A2D_OUT_OF_RESOURCES                     Deprecated constant equal to NASTATUS_A2D_OUT_OF_RESOURCES                    
 * @param NA_A2D_UNKNOWN_IDENTIFIER                   Deprecated constant equal to NASTATUS_A2D_UNKNOWN_IDENTIFIER                  
 * @param NA_A2D_INCORRECT_VERSION                    Deprecated constant equal to NASTATUS_A2D_INCORRECT_VERSION 
 * @param NA_SCRATCHPAD_BAD_ARGUMENT                  Deprecated constant equal to NASTATUS_SCRATCHPAD_BAD_ARGUMENT                 
 * @param NA_SCRATCHPAD_BAD_SIZE                      Deprecated constant equal to NASTATUS_SCRATCHPAD_BAD_SIZE                     
 * @param NA_SCRATCHPAD_DUPLICATE_LOCATION            Deprecated constant equal to NASTATUS_SCRATCHPAD_DUPLICATE_LOCATION           
 * @param NA_SCRATCHPAD_ARRAY_TOO_SMALL               Deprecated constant equal to NASTATUS_SCRATCHPAD_ARRAY_TOO_SMALL              
 * @param NA_SCRATCHPAD_UNKNOWN                       Deprecated constant equal to NASTATUS_SCRATCHPAD_UNKNOWN                      
 * @param NA_SCRATCHPAD_INVALID_HANDLE                Deprecated constant equal to NASTATUS_SCRATCHPAD_INVALID_HANDLE               
 * @param NA_SCRATCHPAD_OUT_OF_BOUNDS                 Deprecated constant equal to NASTATUS_SCRATCHPAD_OUT_OF_BOUNDS                
 * @param NA_SCRATCHPAD_FAILURE                       Deprecated constant equal to NASTATUS_SCRATCHPAD_FAILURE                      
 * @param NA_COMPARATOR_INVALID_PARAMETER             Deprecated constant equal to NASTATUS_COMPARATOR_INVALID_PARAMETER            
 * @param NA_COMPARATOR_NOT_SUPPORTED                 Deprecated constant equal to NASTATUS_COMPARATOR_NOT_SUPPORTED                
 * @param NA_COMPARATOR_INVALID_ID                    Deprecated constant equal to NASTATUS_COMPARATOR_INVALID_ID                   
 *
 * @external
 * @category general_purpose
 * @since 6.2
 * @name DeprecatedNaStatusValues "Deprecated NaStatus values"
 * @include "nastatus.h"
 *
 * @see @link NaStatus
 * @see @link NaStatusValues
*ad*/
#define NA_SUCCESS                                  NASTATUS_SUCCESS
#define NA_STATUS_INVALID_CHANNEL                   NASTATUS_INVALID_CHANNEL    
#define NA_STATUS_INVALID_HANDLE                    NASTATUS_INVALID_HANDLE     
#define NA_STATUS_INVALID_PARAMETER                 NASTATUS_INVALID_PARAMETER  
#define NA_STATUS_RESOURCE_ERROR                    NASTATUS_RESOURCE_ERROR     
#define NA_STATUS_IN_USE                            NASTATUS_IN_USE             
#define NA_STATUS_ENTRY_NOT_FOUND                   NASTATUS_ENTRY_NOT_FOUND      
#define NA_PD_OUT_OF_RESOURCES                      NASTATUS_PD_OUT_OF_RESOURCES                     
#define NA_PD_INVALID_TRIGGER                       NASTATUS_PD_INVALID_TRIGGER                      
#define NA_PD_NO_TRIGGER                            NASTATUS_PD_NO_TRIGGER                           
#define NA_PD_NULL_POINTER                          NASTATUS_PD_NULL_POINTER                         
#define NA_PD_BAD_PORT                              NASTATUS_PD_BAD_PORT                             
#define NA_PD_BAD_DIVISOR                           NASTATUS_PD_BAD_DIVISOR                          
#define NA_PD_BAD_MODE                              NASTATUS_PD_BAD_MODE                             
#define NA_PD_FAILED_TO_SET_INTR                    NASTATUS_PD_FAILED_TO_SET_INTR                   
#define NA_PD_ALREADY_IN_CATNAP                     NASTATUS_PD_ALREADY_IN_CATNAP                    
#define NA_PD_WATCHDOG_ERROR                        NASTATUS_PD_WATCHDOG_ERROR                     
#define NA_IPSEC_OUT_OF_RESOURCES                   NASTATUS_IPSEC_OUT_OF_RESOURCES                  
#define NA_IPSEC_INVALID_PARAMETER                  NASTATUS_IPSEC_INVALID_PARAMETER                 
#define NA_IPSEC_NOT_INITIALIZED                    NASTATUS_IPSEC_NOT_INITIALIZED                   
#define NA_IPSEC_INVALID_PEER                       NASTATUS_IPSEC_INVALID_PEER                      
#define NA_IPSEC_INVALID_KEY                        NASTATUS_IPSEC_INVALID_KEY                       
#define NA_IPSEC_INVALID_ENCRYPTION_ALGORITHM       NASTATUS_IPSEC_INVALID_ENCRYPTION_ALGORITHM      
#define NA_IPSEC_INVALID_KEYLENGTH                  NASTATUS_IPSEC_INVALID_KEYLENGTH                 
#define NA_IPSEC_INVALID_AUTHENTICATION_ALGORITHM   NASTATUS_IPSEC_INVALID_AUTHENTICATION_ALGORITHM  
#define NA_IPSEC_INVALID_HANDLE                     NASTATUS_IPSEC_INVALID_HANDLE                    
#define NA_IPSEC_INVALID_SECONDS                    NASTATUS_IPSEC_INVALID_SECONDS                   
#define NA_IPSEC_INVALID_KILOBYTES                  NASTATUS_IPSEC_INVALID_KILOBYTES                 
#define NA_IPSEC_INVALID_INDEX                      NASTATUS_IPSEC_INVALID_INDEX                     
#define NA_IPSEC_WRONG_LIBRARY                      NASTATUS_IPSEC_WRONG_LIBRARY                     
#define NA_IPSEC_ALREADY_INITIALIZED                NASTATUS_IPSEC_ALREADY_INITIALIZED               
#define NA_IPSEC_INVALID_DEFAULT_POLICY             NASTATUS_IPSEC_INVALID_DEFAULT_POLICY            
#define NA_IPSEC_INVALID_SELECTOR                   NASTATUS_IPSEC_INVALID_SELECTOR                  
#define NA_IPSEC_INVALID_ADDRESS                    NASTATUS_IPSEC_INVALID_ADDRESS                   
#define NA_IPSEC_INVALID_IP_ADDRESS                 NASTATUS_IPSEC_INVALID_IP_ADDRESS                
#define NA_IPSEC_SA_NOT_ACTIVE                      NASTATUS_IPSEC_SA_NOT_ACTIVE                     
#define NA_IPSEC_INVALID_TABLE_ENTRY                NASTATUS_IPSEC_INVALID_TABLE_ENTRY               
#define NA_IPSEC_INVALID_RULES                      NASTATUS_IPSEC_INVALID_RULES                     
#define NA_IPSEC_SA_IN_USE                          NASTATUS_IPSEC_SA_IN_USE                         
#define NA_IPSEC_NOT_CONFIGURED                     NASTATUS_IPSEC_NOT_CONFIGURED                    
#define NA_TCPIP_CANNOT_ADD_ROUTE                   NASTATUS_TCPIP_CANNOT_ADD_ROUTE                  
#define NA_TCPIP_ARP_CACHE_FULL                     NASTATUS_TCPIP_ARP_CACHE_FULL                    
#define NA_EKAHAU_NO_MEMORY                         NASTATUS_EKAHAU_NO_MEMORY                        
#define NA_SNMP_INVALID_POINTER                     NASTATUS_SNMP_INVALID_POINTER                    
#define NA_SNMP_NO_MEMORY                           NASTATUS_SNMP_NO_MEMORY                          
#define NA_SNMP_NOSUCH_TRAPADDR                     NASTATUS_SNMP_NOSUCH_TRAPADDR                    
#define NA_SNMP_CANNOT_SEND_TRAP                    NASTATUS_SNMP_CANNOT_SEND_TRAP                   
#define NA_SNMP_INVALID_DEST_PORT                   NASTATUS_SNMP_INVALID_DEST_PORT                  
#define NA_SNMP_GENERAL_ERROR                       NASTATUS_SNMP_GENERAL_ERROR                      
#define NA_SNMP_TRAPS_MAX_REACHED                   NASTATUS_SNMP_TRAPS_MAX_REACHED                  
#define NA_SNMP_AGENT_LOCKED                        NASTATUS_SNMP_AGENT_LOCKED                       
#define NA_SNMP_INVALID_SYSOID                      NASTATUS_SNMP_INVALID_SYSOID                     
#define NA_SNMP_INVALID_SYSDESCR                    NASTATUS_SNMP_INVALID_SYSDESCR                   
#define NA_SNMP_AGENT_STARTED                       NASTATUS_SNMP_AGENT_STARTED                      
#define NA_SNMP_INVALID_COMMUNITY                   NASTATUS_SNMP_INVALID_COMMUNITY                  
#define NA_SNMP_INVALID_SYSSERV                     NASTATUS_SNMP_INVALID_SYSSERV                    
#define NA_SNMP_AGENT_CANNOT_START                  NASTATUS_SNMP_AGENT_CANNOT_START                 
#define NA_SNMP_INVALID_ENTERPRISE                  NASTATUS_SNMP_INVALID_ENTERPRISE                 
#define NA_SNMP_INVALID_SYSCONTACT                  NASTATUS_SNMP_INVALID_SYSCONTACT                 
#define NA_SNMP_INVALID_SYSLOCATION                 NASTATUS_SNMP_INVALID_SYSLOCATION                
#define NA_SNMP_INVALID_SYSNAME                     NASTATUS_SNMP_INVALID_SYSNAME                    
#define NA_SNMP_INVALID_ARGUMENT                    NASTATUS_SNMP_INVALID_ARGUMENT                   
#define NA_IF_INVALID_IF_NAME                       NASTATUS_IF_INVALID_IF_NAME                      
#define NA_IF_ALREADY_CONFIGURED                    NASTATUS_IF_ALREADY_CONFIGURED                   
#define NA_IF_INTERNAL_ERROR                        NASTATUS_IF_INTERNAL_ERROR                       
#define NA_IF_INVALID_LINK_TYPE                     NASTATUS_IF_INVALID_LINK_TYPE                    
#define NA_IF_INVALID_PARAMETER                     NASTATUS_IF_INVALID_PARAMETER                    
#define NA_IF_ADDR_CONFLICT                         NASTATUS_IF_ADDR_CONFLICT                        
#define NA_IF_NOT_CONFIGURED                        NASTATUS_IF_NOT_CONFIGURED                       
#define NA_IF_CONFIGURATION_IN_PROGRESS             NASTATUS_IF_CONFIGURATION_IN_PROGRESS            
#define NA_AES_MUTEX_FAILED                         NASTATUS_AES_MUTEX_FAILED                        
#define NA_AES_INSTALL_ISR_FAILED                   NASTATUS_AES_INSTALL_ISR_FAILED                  
#define NA_AES_ALREADY_INIT                         NASTATUS_AES_ALREADY_INIT                        
#define NA_AES_NULL_POINTER                         NASTATUS_AES_NULL_POINTER                        
#define NA_AES_INVALID_OPERATION                    NASTATUS_AES_INVALID_OPERATION                   
#define NA_AES_ZERO_LENGTH_BUFFER                   NASTATUS_AES_ZERO_LENGTH_BUFFER                  
#define NA_AES_SEMAPHORE_FAILED                     NASTATUS_AES_SEMAPHORE_FAILED                    
#define NA_AES_INVALID_DMA_DESC_INDEX               NASTATUS_AES_INVALID_DMA_DESC_INDEX              
#define NA_AES_INVALID_KEY_SIZE                     NASTATUS_AES_INVALID_KEY_SIZE                    
#define NA_AES_NOT_INIT                             NASTATUS_AES_NOT_INIT                            
#define NA_AES_INVALID_CCM_L_VALUE                  NASTATUS_AES_INVALID_CCM_L_VALUE                 
#define NA_AES_INVALID_CCM_M_VALUE                  NASTATUS_AES_INVALID_CCM_M_VALUE                 
#define NA_AES_MALLOC_FAILED                        NASTATUS_AES_MALLOC_FAILED                       
#define NA_AES_CBUFFER_FULL                         NASTATUS_AES_CBUFFER_FULL                        
#define NA_AES_OPENSSL_FUNCS_FAILED                 NASTATUS_AES_OPENSSL_FUNCS_FAILED                
#define NA_AES_INVALID_BUFFER_LENGTH                NASTATUS_AES_INVALID_BUFFER_LENGTH               
#define NA_AES_UNALIGNED_BUFFER                     NASTATUS_AES_UNALIGNED_BUFFER                    
#define NA_AES_MAX_XFER_SIZE_EXCEEDED               NASTATUS_AES_MAX_XFER_SIZE_EXCEEDED              
#define NA_A2D_BAD_ARGUMENT                         NASTATUS_A2D_BAD_ARGUMENT                        
#define NA_A2D_BAD_CLOCK_RATE                       NASTATUS_A2D_BAD_CLOCK_RATE                      
#define NA_A2D_BAD_HANDLE                           NASTATUS_A2D_BAD_HANDLE                          
#define NA_A2D_BAD_INPUT                            NASTATUS_A2D_BAD_INPUT                           
#define NA_A2D_BAD_LENGTH                           NASTATUS_A2D_BAD_LENGTH                          
#define NA_A2D_BUFFER_Q_FULL                        NASTATUS_A2D_BUFFER_Q_FULL                       
#define NA_A2D_CHANNEL_IN_USE                       NASTATUS_A2D_CHANNEL_IN_USE                      
#define NA_A2D_DUPLICATE_CALLBACK                   NASTATUS_A2D_DUPLICATE_CALLBACK                  
#define NA_A2D_DUPLICATE_ID                         NASTATUS_A2D_DUPLICATE_ID                        
#define NA_A2D_INVALID_STATE                        NASTATUS_A2D_INVALID_STATE                       
#define NA_A2D_ISR_ALREADY_INSTALLED                NASTATUS_A2D_ISR_ALREADY_INSTALLED               
#define NA_A2D_ISR_NOT_INSTALLED                    NASTATUS_A2D_ISR_NOT_INSTALLED                   
#define NA_A2D_NO_BUFFERS                           NASTATUS_A2D_NO_BUFFERS                          
#define NA_A2D_NO_SUCH_CALLBACK                     NASTATUS_A2D_NO_SUCH_CALLBACK                    
#define NA_A2D_NOT_AVAILABLE                        NASTATUS_A2D_NOT_AVAILABLE                       
#define NA_A2D_NOT_CONFIGURED                       NASTATUS_A2D_NOT_CONFIGURED                      
#define NA_A2D_NOT_RUNNING                          NASTATUS_A2D_NOT_RUNNING                         
#define NA_A2D_NOT_SUPPORTED                        NASTATUS_A2D_NOT_SUPPORTED                       
#define NA_A2D_OUT_OF_RESOURCES                     NASTATUS_A2D_OUT_OF_RESOURCES                    
#define NA_A2D_UNKNOWN_IDENTIFIER                   NASTATUS_A2D_UNKNOWN_IDENTIFIER                  
#define NA_A2D_INCORRECT_VERSION                    NASTATUS_A2D_INCORRECT_VERSION
#define NA_SCRATCHPAD_BAD_ARGUMENT                  NASTATUS_SCRATCHPAD_BAD_ARGUMENT                 
#define NA_SCRATCHPAD_BAD_SIZE                      NASTATUS_SCRATCHPAD_BAD_SIZE                     
#define NA_SCRATCHPAD_DUPLICATE_LOCATION            NASTATUS_SCRATCHPAD_DUPLICATE_LOCATION           
#define NA_SCRATCHPAD_ARRAY_TOO_SMALL               NASTATUS_SCRATCHPAD_ARRAY_TOO_SMALL              
#define NA_SCRATCHPAD_UNKNOWN                       NASTATUS_SCRATCHPAD_UNKNOWN                      
#define NA_SCRATCHPAD_INVALID_HANDLE                NASTATUS_SCRATCHPAD_INVALID_HANDLE               
#define NA_SCRATCHPAD_OUT_OF_BOUNDS                 NASTATUS_SCRATCHPAD_OUT_OF_BOUNDS                
#define NA_SCRATCHPAD_FAILURE                       NASTATUS_SCRATCHPAD_FAILURE                      
#define NA_COMPARATOR_INVALID_PARAMETER             NASTATUS_COMPARATOR_INVALID_PARAMETER            
#define NA_COMPARATOR_NOT_SUPPORTED                 NASTATUS_COMPARATOR_NOT_SUPPORTED                
#define NA_COMPARATOR_INVALID_ID                    NASTATUS_COMPARATOR_INVALID_ID                   






////////////////////////////////////////////////////////////////////////////////////////////
// NDS error codes (see ndsstatus.h)
////////////////////////////////////////////////////////////////////////////////////////////

// Pull in NDS component and error code definitions
#ifdef NDS_SPECIFIC
#include "ndsstatus.h"
#endif

#ifdef __cplusplus
}
#endif 


#endif /* NA_STATUS_H_ */

